/*******************************************************************************
* NXP Semiconductor Inc.
* (c) Copyright 2015 NXP Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. NXP neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE PROJECT
BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES. IN NO EVENT
SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold NXP harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of the advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner             PetrS
* Version           1.0
* Date              Apr-20-2016
* Classification:   General Business Information
* Brief:	    FlexCAN TX/RX demo SW polling
                    based on pin toggle stationery for MPC5777C
********************************************************************************
********************************************************************************
* Detailed Description:
*
* Configures the FlexCAN to transmit and receive a CAN message.
* ECC reporting in the FlexCAN module is enabled.
*
* In this config, CAN_A transmits a message. CAN_B receives the message.
* CAN_A MB8 is configured to send data. CAN_A sends message each 1sec.
* This interval is generated by PIT.
* CAN_B MB9 is configured to receive a message, SW polling is used.
*
* Install jumpers J37 1-2 and J38 1-2
*
* Connect CAN0-CANH on P15-1 to CAN1-CANH on P14-1
* Connect CAN0-CANL on P15-2 to CAN1-CANL on P14-2
* 
* ------------------------------------------------------------------------------
* Test HW:         MPC5777C-512DS Rev.A + MPC57xx MOTHER BOARD Rev.C
* MCU:             PPC5777CMM03 2N45H CTZZS1521A
* Fsys:            PLL1 = core_clk = 264MHz, PLL0 = 192MHz
* Debugger:        Lauterbach Trace32
* Target:          internal_FLASH
* Terminal:        19200-8-no parity-1 stop bit-no flow control on eSCI_A
*		   use USB connector (J21) on minimodule
*
* EVB connection:  ETPUA30 (PortP P23-15) --> USER_LED_1 (P7-1)
*                  ETPUA31 (PortP P23-14) --> USER_LED_2 (P7-2)
*
*
********************************************************************************
Revision History:
Ver  Date         Author            Description of Changes
0.1  Aug-26-2015  David Tosenovjan  Initial version
0.2  Sep-01-2015  David Tosenovjan  printf added, minor edits
0.3  Nov-26-2015  David Tosenovjan  - lauterbach scripts changed
                                    - multicore added
                                    - intc_sw_handlers.s changed
                                    - Interrupts.c changed
                                    - other edits
1.0  Apr-20-2016  PetrS		    - Added FlexCAN related code				    
*******************************************************************************/
#include <ppc_ghs.h>

#include "MPC577xC.h"
#include "Interrupts.h"
#include "uart.h"
#include "stdio.h"

/*******************************************************************************
* External objects
*******************************************************************************/

/* Prototype for PRC1 startup */
#ifdef __cplusplus
    extern "C"{void __start();}
#else
    extern void __start();
#endif


/*******************************************************************************
* Global variables
*******************************************************************************/
uint8_t   u8_txdata = 0; /* tranmitted data */
uint8_t   u8_rxdata = 0; /* received data */

/*******************************************************************************
* Constants and macros
*******************************************************************************/
/* FlexCAN module register's bit masks */
#define CAN_MCR_MDIS        0x80000000
#define CAN_MCR_FRZ         0x40000000
#define CAN_MCR_FEN         0x20000000
#define CAN_MCR_HALT        0x10000000
#define CAN_MCR_NOTRDY      0x08000000
#define CAN_MCR_SOFTRST     0x02000000
#define CAN_MCR_FRZACK      0x01000000
#define CAN_MCR_WRNEN       0x00200000
#define CAN_MCR_LPMACK      0x00100000
#define CAN_MCR_SRXDIS      0x00020000
#define CAN_MCR_BCC         0x00010000
#define CAN_MCR_LPRIOEN     0x00002000
#define CAN_MCR_AEN         0x00001000

// SIU macros
#define APC 0x2000
#define OBE 0x0200
#define IBE 0x0100
#define ODE 0x0020
#define SRC 0x0004
#define WPE 0x0002
#define WPS 0x0001
#define ALT0 0x0000
#define ALT1 0x0400
#define ALT2 0x0800
#define ALT3 0x0C00

#define ETPUA30_pin 144 // PCR number
#define ETPUA31_pin 145
#define EMIOS0_ch0_pin 179
#define EMIOS0_ch16_pin 195

#define LED1_pin ETPUA30_pin
#define LED2_pin ETPUA31_pin

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void HW_init(void);
static void SIU_Init(void);
static void PIT0_Init(void);
static void Terminal_ECHO(void);
void PIT0_ISR(void);



/*******************************************************************************
* Local variables
*******************************************************************************/

/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : HW_init
Engineer      : David Tosenovjan
Date          : Aug-26-2015
Parameters    : 
Modifies      : 
Returns       : 
Notes         : initialization of the hw for the purposes of this example
Issues        : 
*******************************************************************************/
static void HW_init(void)
{
    /* set round robin for all slaves */
    XBAR.SGPCR0.B.ARB = 1;
    XBAR.SGPCR1.B.ARB = 1;
    XBAR.SGPCR2.B.ARB = 1;
    XBAR.SGPCR3.B.ARB = 1;
    XBAR.SGPCR4.B.ARB = 1;
    XBAR.SGPCR6.B.ARB = 1;
    XBAR.SGPCR7.B.ARB = 1;
    
    /* initialize GPIOs */
    SIU_Init();
}


/*******************************************************************************
Function Name : SIU_Init
Engineer      : PetrS
Date          : Apr-20-2016
Parameters    : 
Modifies      : 
Returns       : 
Notes         : 
Issues        : 
*******************************************************************************/
static void SIU_Init(void)
{
    SIU.PCR[LED1_pin].R = ALT0 | OBE;    // general purpose output
    SIU.PCR[LED2_pin].R = ALT0 | OBE;    // general purpose output
}


/*******************************************************************************
Function Name : PIT0_Init
Engineer      : PetrS
Date          : Apr-20-2016
Parameters    : 
Modifies      : 
Returns       : 
Notes         : set for 1sec delay assuming 96MHz peripheral clock
Issues        : 
*******************************************************************************/
static void PIT0_Init(void)
{
    /* 30: MDIS = 0 to enable clock for PITs. */
    /* 31: FRZ = 1 for Timers stopped in debug mode */
    PIT.MCR.R = 0x00000001;    
    PIT.CH[0].LDVAL.R = 96000000 - 1;
        
    /* clear the TIF flag */
    PIT.CH[0].TFLG.R = 0x00000001;
        
    /* 31: TEN = 1 for timer active */
    PIT.CH[0].TCTRL.R = 0x00000001;
}


/*******************************************************************************
Function Name : Terminal_ECHO
Engineer      : David Tosenovjan
Date          : Aug-26-2015
Parameters    : 
Modifies      : 
Returns       : 
Notes         :                          
Issues        : 
*******************************************************************************/
static void Terminal_ECHO(void)
{
    char ch;
    
    if (ESCI_A.SR.B.RDRF == 0)
        return;
    else 
    {
        ESCI_A.SR.R = 0x20000000; // clear RDRF flag
        ch =(unsigned char) ESCI_A.DR.B.D; // read byte of Data
        switch(ch)
        {
            // enter
            case '\n':
            case '\r':
                printf("\r\n");
                break;
                    
            // other symbols
            default:
                printf("%c",ch); 
                fflush(stdout);
        } 
    } 
}     

/*******************************************************************************
Function Name : FlexCAN_A_Init
Engineer      : PetrS
Date          : Mar-17-2015
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : FlexCAN0 initialization
Issues        : NONE
*******************************************************************************/
void FlexCAN_A_Init(void)
{
#define CAN_A_ADDR 0xFFFC0000UL

    uint32_t i = 0;
    
    /* enable the FlexCAN module, reset and freeze */
    CAN_A.MCR.R = (0
                    | CAN_MCR_FRZ  /* enabled to enter Freeze mode */ 
                    | CAN_MCR_HALT /* enter freeze mode if FRZ bit is set */
                    | CAN_MCR_SOFTRST /* soft reset */
                    | CAN_MCR_SRXDIS  /* self reception enabled */
                    | CAN_MCR_BCC  /* individual Rx masking and queue */
                    | 0x0000003F);                      
                    
                    
    /* wait until soft reset completes */
    while(1 == CAN_A.MCR.B.SOFTRST) {};
    
    /* double check that we are actually in freeze mode */
    while(0 == CAN_A.MCR.B.FRZACK) {};
    while(0 == CAN_A.MCR.B.NOTRDY) {};

    CAN_A.CR.R = 0x04E40004;	/* CAN bus: 40 MHz clksrc, 500K bps with 16 tq */
                              /* PRESDIV+1 = Fclksrc/Ftq = 40 MHz/8MHz = 5 */
                              /*    so PRESDIV = 4 */
                              /* PSEG2 = Phase_Seg2 - 1 = 5 - 1 = 4 */
                              /* PSEG1 = PSEG2 = 4 */
                              /* PROPSEG= Prop_Seg - 1 = 5 - 1 = 4 */
                              /* RJW = Resync Jump Width - 1 = 4 - 1 = 1 */
                              /* SMP = 0: use 1 bit per CAN sample */
                              /* CLKSRC=0 (unchanged): Fcanclk= Fxtal= 40 MHz*/
   
    CAN_A.CTRL2.B.WRMFRZ = 1; // Enable Unrestricted Write Access to FlexCAN Memory
	 
    (*(volatile long long int *) (CAN_A_ADDR + 0xAB0))=0x0000000000000000; //TX_SMB
    (*(volatile long long int *) (CAN_A_ADDR + 0xAB8))=0x0000000000000000; //TX_SMB

    (*(volatile long long int *) (CAN_A_ADDR + 0xAC0))=0x0000000000000000; //RX_SMB0
    (*(volatile long long int *) (CAN_A_ADDR + 0xAC8))=0x0000000000000000; //RX_SMB0

    (*(volatile long long int *) (CAN_A_ADDR + 0xAD0))=0x0000000000000000; //RX_SMB1
    (*(volatile long long int *) (CAN_A_ADDR + 0xAD8))=0x0000000000000000; //RX_SMB1

    (*(volatile long int *) (CAN_A_ADDR + 0xA80))=0x00000000; //RX_FIR0
    (*(volatile long int *) (CAN_A_ADDR + 0xA84))=0x00000000; //RX_FIR1

    (*(volatile long int *) (CAN_A_ADDR + 0xA88))=0x00000000; //RX_FIR2
    (*(volatile long int *) (CAN_A_ADDR + 0xA8C))=0x00000000; //RX_FIR3

    (*(volatile long int *) (CAN_A_ADDR + 0xA90))=0x00000000; //RX_FIR4
    (*(volatile long int *) (CAN_A_ADDR + 0xA94))=0x00000000; //RX_FIR5

    (*(volatile long int *) (CAN_A_ADDR + 0xA98))=0x00000000; //Reserved Location 0xA98 to 0xA9B
    (*(volatile long int *) (CAN_A_ADDR + 0xA9C))=0x00000000; //Reserved Location 0xA9C to 0xA9F

    for(i=0;i<128;i++)
	(*(volatile long long int *) (CAN_A_ADDR + 0x480 + (i*8))) = 0x0000000000000000; //Reserved Location from 0x480 to 0x87F

    for(i=0;i<32;i++)
	(*(volatile long long int *) (CAN_A_ADDR + 0x980 + (i*8))) = 0x0000000000000000; //Reserved Location from 0x980 to 0xA7F


    CAN_A.CTRL2.B.WRMFRZ = 0; // Disable Unrestricted Write Access to FlexCAN Memory
    
    /* Make first 32 message buffers inactive by writing their control fields 
     * to "not active". They will be left
     * inactive until we're ready for communication. */      
    for(i=0;i<64;i++)
    {
        CAN_A.BUF[i].CS.R = 0;
	CAN_A.BUF[i].ID.R = 0;
	CAN_A.BUF[i].DATA.W[0] = 0;
	CAN_A.BUF[i].DATA.W[1] = 0;
    }

    CAN_A.RXGMASK.R = 0x1FFFFFFF;
    CAN_A.RX14MASK.R = 0x1FFFFFFF;
    CAN_A.RX15MASK.R = 0x1FFFFFFF;
    
    /* set mask registers - all ID bits must match */
    for(i=0;i<64;i++)
    {
        CAN_A.RXIMR[i].R = 0x1FFFFFFF;
    }

    /* configure CNTX_A and CNRX_A pin functions on PCR83 and PCR84 */
    SIU.PCR[83].R = 0x060C; /* CNTXA, push/pull, max slew rate */
    SIU.PCR[84].R = 0x0503; /* CNRXA, weak pull device disabled */
    
    /* Finally clear the HALT flag in MCR to enable the FlexCAN
     * to synchronize with the CAN bus and allow
     * participation in communication. */
    CAN_A.MCR.B.HALT = 0;
    
    /* wait until FlexCAN ready */
    while(1 == CAN_A.MCR.B.FRZACK) {};
    while(1 == CAN_A.MCR.B.NOTRDY) {};
}

/*******************************************************************************
Function Name : FlexCAN_B_Init
Engineer      : PetrS
Date          : Mar-17-2015
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : FlexCAN1 initialization
Issues        : NONE
*******************************************************************************/
void FlexCAN_B_Init(void)
{
#define CAN_B_ADDR 0xFFFC4000UL

    uint32_t i = 0;
    
    /* enable the FlexCAN module, reset and freeze */
    CAN_B.MCR.R = (0
                    | CAN_MCR_FRZ  /* enabled to enter Freeze mode */ 
                    | CAN_MCR_HALT /* enter freeze mode if FRZ bit is set */
                    | CAN_MCR_SOFTRST /* soft reset */
                    | CAN_MCR_SRXDIS  /* self reception enabled */
                    | CAN_MCR_BCC  /* individual Rx masking and queue */
                    | 0x0000003F);                      
                    
                    
    /* wait until soft reset completes */
    while(1 == CAN_B.MCR.B.SOFTRST) {};
    
    /* double check that we are actually in freeze mode */
    while(0 == CAN_B.MCR.B.FRZACK) {};
    while(0 == CAN_B.MCR.B.NOTRDY) {};

    CAN_B.CR.R = 0x04E40004;	/* CAN bus: 40 MHz clksrc, 500K bps with 16 tq */
                              /* PRESDIV+1 = Fclksrc/Ftq = 40 MHz/8MHz = 5 */
                              /*    so PRESDIV = 4 */
                              /* PSEG2 = Phase_Seg2 - 1 = 5 - 1 = 4 */
                              /* PSEG1 = PSEG2 = 4 */
                              /* PROPSEG= Prop_Seg - 1 = 5 - 1 = 4 */
                              /* RJW = Resync Jump Width - 1 = 4 - 1 = 1 */
                              /* SMP = 0: use 1 bit per CAN sample */
                              /* CLKSRC=0 (unchanged): Fcanclk= Fxtal= 40 MHz*/
   

    

    CAN_B.CTRL2.B.WRMFRZ = 1; // Enable Unrestricted Write Access to FlexCAN Memory
	 
    (*(volatile long long int *) (CAN_B_ADDR + 0xAB0))=0x0000000000000000; //TX_SMB
    (*(volatile long long int *) (CAN_B_ADDR + 0xAB8))=0x0000000000000000; //TX_SMB

    (*(volatile long long int *) (CAN_B_ADDR + 0xAC0))=0x0000000000000000; //RX_SMB0
    (*(volatile long long int *) (CAN_B_ADDR + 0xAC8))=0x0000000000000000; //RX_SMB0

    (*(volatile long long int *) (CAN_B_ADDR + 0xAD0))=0x0000000000000000; //RX_SMB1
    (*(volatile long long int *) (CAN_B_ADDR + 0xAD8))=0x0000000000000000; //RX_SMB1

    (*(volatile long int *) (CAN_B_ADDR + 0xA80))=0x00000000; //RX_FIR0
    (*(volatile long int *) (CAN_B_ADDR + 0xA84))=0x00000000; //RX_FIR1

    (*(volatile long int *) (CAN_B_ADDR + 0xA88))=0x00000000; //RX_FIR2
    (*(volatile long int *) (CAN_B_ADDR + 0xA8C))=0x00000000; //RX_FIR3

    (*(volatile long int *) (CAN_B_ADDR + 0xA90))=0x00000000; //RX_FIR4
    (*(volatile long int *) (CAN_B_ADDR + 0xA94))=0x00000000; //RX_FIR5

    (*(volatile long int *) (CAN_B_ADDR + 0xA98))=0x00000000; //Reserved Location 0xA98 to 0xA9B
    (*(volatile long int *) (CAN_B_ADDR + 0xA9C))=0x00000000; //Reserved Location 0xA9C to 0xA9F

    for(i=0;i<128;i++)
	(*(volatile long long int *) (CAN_B_ADDR + 0x480 + (i*8))) = 0x0000000000000000; //Reserved Location from 0x480 to 0x87F

    for(i=0;i<32;i++)
	(*(volatile long long int *) (CAN_B_ADDR + 0x980 + (i*8))) = 0x0000000000000000; //Reserved Location from 0x980 to 0xA7F


    CAN_B.CTRL2.B.WRMFRZ = 0; // Disable Unrestricted Write Access to FlexCAN Memory

    
    /* Make first 32 message buffers inactive by writing their control fields 
     * to "not active". They will be left
     * inactive until we're ready for communication. */      
    for(i=0;i<64;i++)
    {
        CAN_B.BUF[i].CS.R = 0;
	CAN_B.BUF[i].ID.R = 0;
	CAN_B.BUF[i].DATA.W[0] = 0;
	CAN_B.BUF[i].DATA.W[1] = 0;
    }

    CAN_B.RXGMASK.R = 0x1FFFFFFF;
    CAN_B.RX14MASK.R = 0x1FFFFFFF;
    CAN_B.RX15MASK.R = 0x1FFFFFFF;
    
    /* set mask registers - all ID bits must match */
    for(i=0;i<64;i++)
    {
        CAN_B.RXIMR[i].R = 0x1FFFFFFF;
    }
    
    /* configure CNTX_B and CNRX_B pin functions on PCR85 and PCR86 */
    SIU.PCR[85].R = 0x060C; /* CNTXA, push/pull, max slew rate */
    SIU.PCR[86].R = 0x0503; /* CNRXA, weak pull device disabled */
    
    /* Finally clear the HALT flag in MCR to enable the FlexCAN
     * to synchronize with the CAN bus and allow
     * participation in communication. */
    CAN_B.MCR.B.HALT = 0;
    
    /* wait until FlexCAN ready */
    while(1 == CAN_B.MCR.B.FRZACK) {};
    while(1 == CAN_B.MCR.B.NOTRDY) {};
}

/*******************************************************************************
Function Name : TransmitMsg
Engineer      : PetrS
Date          : Mar-17-2015
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : FlexCAN0 simple transmit. per AN2865
Issues        : NONE
*******************************************************************************/
static void TransmitMsg(void) 
{
 
  /* Assumption:  Message buffer CODE is INACTIVE */
  /* prepare data to send */                
  /* prepare data to send */                
  u8_txdata ^= 0x01; /* toggle bit 0 */
            
  /* CAN_A MB8 Tx Send Once, data length to send = 1 byte */
  CAN_A.BUF[8].CS.R = 0x08600000; /* Tx INACTIVE + IDE + SRR */
  CAN_A.BUF[8].ID.R = 0x12340000; /* Ext.ID = 0x12340000 */
  CAN_A.BUF[8].DATA.B[0] = u8_txdata; /* data to send */

  CAN_A.BUF[8].CS.R = 0x0C610000; /* send 1 byte, ext.frame */

  while ( CAN_A.IFRL.B.BUF08I  == 0) { } // wait until the tx is completed.
  CAN_A.IFRL.R = 0x00000100; 
}

/*******************************************************************************
Function Name : ReceiveMsg
Engineer      : PetrS
Date          : Mar-17-2015
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : FlexCAN1 simple receive. per AN2865
Issues        : NONE
*******************************************************************************/
static void ReceiveMsg(void) 
{
  uint32_t temp = 0;      /* temporary and volatile 32bit */
 	
	/* upon receiving the interrupt flag */

    /* mandatory - read control/status word - lock the MB */
	temp = CAN_B.BUF[9].CS.R;
	/* check the ID field */
	/* not really needed in this example */
	/* useful in case when multiple IDs are received
	 * by one message buffer (Rx mask feature) */ 
	temp = CAN_B.BUF[9].ID.R;
	if(0x12340000 != temp)
	{
	    /* error ! trap here.
	     * we have configured this MB to receive 
	     * just ID 0x12340000 and not another */
	    while(1) {}
	}

	/* read the received data */
	u8_rxdata = CAN_B.BUF[9].DATA.B[0];

	/* release the internal lock for all Rx MBs
	 * by reading the TIMER */
	temp = CAN_B.TIMER.R; 

	/* do someting with the received data */
    /* drive the output pin with the received data */
    SIU.GPDO[LED1_pin].R = u8_rxdata; 
}
/*******************************************************************************
* Global functions
*******************************************************************************/

/*******************************************************************************
Function Name : main
Engineer      : PetrS
Date          : Apr-20-2016
Parameters    : 
Modifies      : 
Returns       : 
Notes         :                             
Issues        : 
*******************************************************************************/
void main (void)
{    
    vint32_t counter = 0, i;
    
    /* Hardware initialization */
    HW_init();

    PIT0_Init();	// set to generate 1sec interval
    FlexCAN_A_Init();
    FlexCAN_B_Init();
    
    Interrupts_init();
    Interrupts_enable_Core0();

   // printf("MPC5777C: FlexCAN test\r\n");
   // printf("\n\r");
    
    /* Start the second core */ 
    SIU.RSTVEC1.R = (unsigned long)__start | 0x00000001; // VLE
    //SIU.RSTVEC1.R = (unsigned long)__start; // Book-E

    /* subscribe FlexCAN_B's MB9 for Rx - extended frame format */
    CAN_B.BUF[9].CS.R = 0x00200000; /* Rx inactive + IDE */
    CAN_B.BUF[9].ID.R = 0x12340000; /* Ext.ID = 0x12340000 */
    CAN_B.BUF[9].CS.R = 0x04200000; /* Rx active and empty + IDE */
    
    /* Loop forever */
    while (1) 
    {
	
	if(PIT.CH[0].TFLG.B.TIF)
    	{
	    TransmitMsg();	/* Transmit one message from a FlexCAN 0 buffer */
	    /* if nothing connected to the bus, message is transmitting repeatedly 
	       (not acknownledged) and a MCU is going to the error passive mode */

	    PIT.CH[0].TFLG.B.TIF = 1;     // Clear PIT_0 interrupt flag
  	}
	if(CAN_B.IFRL.B.BUF09I) 	// test BUF9I flag if set
        {
	    ReceiveMsg(); 

	    /* clear the BUF09I bit - bit 9 */
	    CAN_B.IFRL.R = 0x00000200;

        }

	i++;
#if 0
	/* delay */
        while(counter < 500000)
        {
            counter++;
            Terminal_ECHO();

        }  
        counter = 0;      
#endif
    }
    
}
