/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2015 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE 
SERVICES. IN NO EVENT SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL 
DAMAGES ARISING OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             uart.c
* Owner:            David Tosenovjan
* Version:          0.1
* Date:             Aug-26-2015
* Classification:   General Business Information
* Brief:            terminal IO
********************************************************************************
* Detailed Description: 
* 
* Implements GHS ind_io library function calls to do printf() on eSCI_A
* Functions "read" and "write" defined in this module replace the ones from the
* library so finally content of printf() is sent to eSCI_A.
*
* ------------------------------------------------------------------------------
* Test HW:   MPC5777C-512DS Rev.A + MPC57xx MOTHER BOARD Rev.C
* MCU:       PPC5777CMM03 2N45H CTZZS1521A
* Terminal:  19200-8-no parity-1 stop bit-no flow control on eSCI_A 
* Fsys:      PLL0 = 192MHz
*            PLL1 = core_clk = 264MHz
*            per_clk = PLL0/2 = 96MHz (eSCI clock)
*
********************************************************************************
Revision History:
Ver  Date         Author            Description of Changes
0.1  Aug-26-2015  David Tosenovjan  Initial version
*******************************************************************************/

#include "MPC577xC.h"
#include "uart.h"


/*******************************************************************************
* Global variables
*******************************************************************************/
int UART_initialized = 0;

/*******************************************************************************
* Constants and macros
*******************************************************************************/

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/

/*******************************************************************************
* Local variables
*******************************************************************************/

/*******************************************************************************
* Local functions
*******************************************************************************/
 
/*******************************************************************************
Function Name : Init_eSCI_A
Engineer      : David Tosenovjan
Date          : 
Parameters    : 
Modifies      : 
Returns       : 
Notes         : initializes eSCI_A module for UART mode
Issues        : expecting per_clk = 96 MHz (PLL0/2 = 192/2)
*******************************************************************************/
void Init_eSCI_A (void) 
{
	
	SIU.PCR[89].R = 0x400;  // Configure pad for primary func: TxDA
	SIU.PCR[90].R = 0x400;  // Configure pad for primary func: RxDA
	
	/* note: SBR = fsys/(16 * desired baudrate) < 8191 */

	ESCI_A.CR2.R = 0x2000;  // module is enabled (default setting)
    
    /* per_clk = 96MHz */
    ESCI_A.CR1.B.SBR = 313; // 19200 baud, 8 bits, no parity
	ESCI_A.CR1.B.TE = 1;    // Tx enabled
	ESCI_A.CR1.B.RE = 1;    // Rx enabled	
    
}


/*******************************************************************************
Function Name : TransmitData
Engineer      : 
Date          : 
Parameters    : pBuf - input string. won't be modified by the function
              : cnt  - input number of characters to be transmitted
Modifies      : 
Returns       : 
Notes         : Tx data on eSCI_A. polled mode. 
Issues        :
*******************************************************************************/
void TransmitData(const char * const pBuf, uint32_t cnt) 
{
    uint8_t	j = 0;                                 /* Dummy variable */
    for (j=0; j< cnt; j++) 
    {  /* Loop for character string */
    
        /* Wait for transmit data reg empty = 1 */
        while (ESCI_A.SR.B.TDRE == 0) {}       
        ESCI_A.SR.R = 0x80000000;              /* Clear TDRE flag */
        ESCI_A.DR.B.D = *(pBuf+j);          /* Transmit 8 bits Data */        
    }
}

/*******************************************************************************
Function Name : ReceiveData
Engineer      : 
Date          : 
Parameters    : pBuf - address of a char where the received char is written to
                       the address (pBuf) doesn't change in the function
Modifies      : 
Returns       : 
Notes         : Rx data on eSCI_A. polled mode. 
Issues        : 
*******************************************************************************/
void ReceiveData(char *pBuf)
{
    while (ESCI_A.SR.B.RDRF == 0) {}    /* Wait for receive data reg full = 1 */
    ESCI_A.SR.R = 0x20000000;           /* Clear RDRF flag */
    *pBuf = (char)ESCI_A.DR.B.D;               /* Read byte of Data*/  
}


/*******************************************************************************
* Global functions
*******************************************************************************/ 

/*
Methods called by GHS ind_io libraries to perform console IO:
*/


int read (int fno, void *buf, int size)
{
    int count;
    
    if (UART_initialized == 0)
    {
        Init_eSCI_A();
        UART_initialized = 1;
    }

    for (count = 0; count < size; count++)
    {
        ReceiveData( (char *)buf + count );
    }

    return size;
}


int write (int fno, const void *buf, int size)
{ 
    if (UART_initialized == 0)
    {
        Init_eSCI_A();
        UART_initialized = 1;
    }

    TransmitData ((const char *)buf,size);
    return size; 
}


